@@ -20,6 +20,8 @@ extern void group_callback();
 
 void pixmap_selection(Widget);
 
+extern int childs_open;
+
 char TOOLBAR_CONFIG_FILE[255];
 int START_X;
 int START_Y;
@@ -32,7 +34,8 @@ void button_event( Widget button, XtPointer client_data, XEvent *event)
     XButtonPressedEvent *bevent = (XButtonPressedEvent *) event;
 
     if(bevent->button != 3)
- 	return; 
+ 	return;
+
     XmMenuPosition(popup, bevent);
     XtManageChild(popup);
 
@@ -53,9 +56,12 @@ void show_help(Widget button, XtPointer client_data, XEvent
   if(cevent->type != EnterNotify)
     return;
 
+
   if(popup_frame)
     return;
 
+
+
   toolbar = start;
   while(toolbar->button != button)
     toolbar = toolbar->next;
@@ -194,9 +200,12 @@ void delete_help(Widget button, XtPointer client_data, XEvent
   if(cevent->type != LeaveNotify)
     return;
 
+
   if(!popup_frame)
     return;
 
+
+
   toolbar = start;
   while(toolbar->button != button)
     toolbar = toolbar->next;
@@ -215,7 +224,6 @@ Widget add_new_button(Widget parent, Widget button, int button_type)
 {
     Widget new_button, popup, row_col, group;
     Pixmap pixmap;
-    int width, height;
     int pos;
     Position x_pos, y_pos;
     tool_button *help, *help_2;
@@ -274,13 +282,17 @@ Widget add_new_button(Widget parent, Widget button, int button_type)
 	      button_num++;
 	    
 	    pos = XmLAST_POSITION;
+
 	    if((position == TOPRIGHT) && (orientation == XmHORIZONTAL))
 		pos = 0;
 	    if(position == BOTTOMRIGHT)
 		pos = 0;
 	    if((position == BOTTOMLEFT) && (orientation == XmVERTICAL))
 		pos = 0;
-		
+	    if((position == CUSTOM) && (orientation == XmHORIZONTAL)
+	       && (START_X > (screen_width/2)))
+	       pos = 0;
+
 	    new_button = XtVaCreateManagedWidget("Button",
 						 xmPushButtonWidgetClass,
 						 parent,
@@ -403,7 +415,7 @@ Widget add_new_button(Widget parent, Widget button, int button_type)
 						  NULL);
 
 		toolbar->rowcol = row_col;
-		strcpy(toolbar->app_path, "Group Button");
+		strcpy(toolbar->description, "Group Button");
 	      }
 
 	    XtAddEventHandler(new_button, ButtonPressMask, False, (XtEventHandler)button_event, popup);    
@@ -419,6 +431,7 @@ Widget add_new_button(Widget parent, Widget button, int button_type)
 			      NULL);
 	      }
 
+
 	    XtVaGetValues(toplevel, 
 			  XmNx, 
 			  &x_pos, 
@@ -433,19 +446,32 @@ Widget add_new_button(Widget parent, Widget button, int button_type)
 	    
 	    if(((y_pos+window_height) > screen_height)&&(orientation==XmVERTICAL)) 
 	      y_pos = screen_height-window_height; 
+
 	    if(((x_pos+window_width) > screen_width)&&(orientation==XmHORIZONTAL)) 
 	      x_pos = screen_width-window_width;   
 	    
-	    XtVaSetValues(toplevel,  
-			  XmNx,
-			  x_pos,
-			  XmNy,
-			  y_pos,
-			  NULL );
+	    
+	      
+	      
+	    if((position == CUSTOM) && (orientation == XmHORIZONTAL)
+	       && (START_X > (screen_width/2)))
+	      XtVaSetValues(toplevel,  
+			    XmNx,
+			    START_X - window_width + button_x,
+			    XmNy,
+			    y_pos,
+			    NULL );
+	    else
+	      XtVaSetValues(toplevel,  
+			    XmNx,
+			    x_pos,
+			    XmNy,
+			    y_pos,
+			    NULL );
 
 	    if(XtIsManaged(XtParent(toolbar->rowcol)))
 	       {
-		 Dimension sub_width, sub_height, b_height, b_width;
+		 Dimension b_height, b_width;
 		 Position sub_x, sub_y;
 
 		 XtVaGetValues(new_button,
@@ -454,11 +480,8 @@ Widget add_new_button(Widget parent, Widget button, int button_type)
 			       XmNheight,
 			       &b_height,
 			       NULL);
+
 		 XtVaGetValues(XtParent(toolbar->rowcol),
-			       /*XmNheight,
-			       &sub_height,
-			       XmNwidth,
-			       &sub_width,*/
 			       XmNx,
 			       &sub_x,
 			       XmNy,
@@ -484,6 +507,7 @@ Widget add_new_button(Widget parent, Widget button, int button_type)
 				 NULL);
 
 	       }
+
 	    if(button_type != GROUP)
 	      XtAddCallback(new_button, XmNactivateCallback,
 			    app_callback, NULL);
@@ -493,12 +517,15 @@ Widget add_new_button(Widget parent, Widget button, int button_type)
 	    
 	    XtAddEventHandler(new_button, EnterWindowMask, False,
 			     (XtEventHandler) show_help, NULL);
+
 	    XtAddEventHandler(new_button, LeaveWindowMask, False,
 			     (XtEventHandler) delete_help, NULL);
+
 	    toolbar->type = button_type;
 	    
 	    return new_button;
 	}
+    return NULL;
 }
 
 void select_file(Widget file, XtPointer client_data, XtPointer
@@ -524,14 +551,14 @@ void file_ok(Widget file, XtPointer client_data, XtPointer
 		      call_data)
 {
     char *filename;
-    Dimension height, width;
-    int depth;
     Widget button;
     Position x_pos, y_pos;
 
     XmFileSelectionBoxCallbackStruct *cbs =
 	(XmFileSelectionBoxCallbackStruct *) call_data;
 
+    childs_open--;
+
     if(!XmStringGetLtoR(cbs->value, XmFONTLIST_DEFAULT_TAG, &filename))
 	return;
 
@@ -616,8 +643,10 @@ void file_ok(Widget file, XtPointer client_data, XtPointer
 void file_cancel(Widget file, XtPointer client_data, XtPointer
 		      call_data)
 {
-    XtDestroyWidget(file);
-    pixmap_dialog = NULL;
+  childs_open--;
+  
+  XtDestroyWidget(file);
+  pixmap_dialog = NULL;
 }
 
 
@@ -633,6 +662,8 @@ void pixmap_selection(Widget button)
     if(pixmap_dialog)
       return;
 
+    childs_open++;
+
     toolbar = start;
     while(toolbar->button != button)
 	toolbar = toolbar->next;
@@ -657,6 +688,8 @@ void pixmap_selection(Widget button)
 
 
     XtVaSetValues(pixmap_dialog,
+		  XmNdialogStyle,
+		  XmDIALOG_PRIMARY_APPLICATION_MODAL,
 		  XmNpattern,
 		  XmStringCreateLocalized("*.xpm"),
 		  XmNdirectory,
@@ -705,7 +738,6 @@ void save_config(Widget parent)
     char filename[MAXPATH];
     Widget message_box;
     int counter = 0;
-    XmString message;
 
     strcpy(filename, getenv("HOME"));
     strcat(filename, "/.toolbar");
@@ -774,18 +806,113 @@ void save_config(Widget parent)
 	write(config,(char *) &quick_info, sizeof(int));
         write(config,(char *) &START_X, sizeof(int));
         write(config,(char *) &START_Y, sizeof(int));
+	write(config,(char *) &auto_hide, sizeof(int));
 	close(config);
       }
     
 }
 
-    
+
+void export_it(Widget parent, XtPointer client_data, XtPointer
+		     call_data)
+{
+  char *filename;
+  tool_button *help;
+  FILE *fd;
+  Widget message_box;
+
+  XmFileSelectionBoxCallbackStruct *cbs =
+    (XmFileSelectionBoxCallbackStruct *) call_data;
+
+  childs_open--;
+
+  if(!XmStringGetLtoR(cbs->value, XmFONTLIST_DEFAULT_TAG, &filename))
+    return;
+
+  if((fd = fopen(filename, "w+")) == NULL)
+    {
+      message_box = XmCreateInformationDialog(parent,
+					      "Message_Box",
+					      NULL,
+					      0);
+      XtVaSetValues(message_box, 
+		    XmNmessageString,
+		    XmStringCreateLocalized("Sorry, I am not able to serve you master" ),
+		    XmNdialogTitle,
+		    XmStringCreateLocalized("OOps!"),
+		    NULL);
+      
+      XtUnmanageChild(XmMessageBoxGetChild(message_box,
+					   XmDIALOG_CANCEL_BUTTON));
+      XtUnmanageChild(XmMessageBoxGetChild(message_box, XmDIALOG_HELP_BUTTON));
+      XtManageChild(message_box);	
+    }
+  else
+    {
+      help = start;
+      while(help != NULL)
+	{
+	  fprintf(fd, "%s\n", help->description);
+	  fprintf(fd, "%s\n", help->pixmap_path);
+	  fprintf(fd, "%s\n", help->app_path);
+	  fprintf(fd, "%s\n", help->parameters);
+	  fprintf(fd, "---------------------------------\n");
+	  help = help->next;
+	}
+      
+      fclose(fd);
+    }
+
+  XtDestroyWidget(parent);
+  XtFree(filename);
+
+}
+
+void export_cancel(Widget parent, XtPointer client_data, XtPointer
+		     call_data)
+{
+  childs_open--;
+  XtDestroyWidget(parent);
+}
+
+/* export writes all data of the structure into a given file */
+void export(Widget parent)
+{
+  Widget file;
+  char path[MAXPATH];
+
+  childs_open++;
+
+  file = XmCreateFileSelectionDialog(parent,
+				     "P",
+				     NULL,
+				     0);
+
+  strcpy(path, getenv("HOME"));
+
+  XtVaSetValues(file,
+		XmNdialogStyle,
+		XmDIALOG_PRIMARY_APPLICATION_MODAL,
+		XmNpattern,
+		XmStringCreateLocalized("*"),
+		XmNdirectory,
+		XmStringCreateLocalized(path),
+		XmNdialogTitle,
+		XmStringCreateLocalized("Export"),
+		XmNresizePolicy,
+		XmRESIZE_ANY,
+		NULL);
+  
+  XtAddCallback(file, XmNcancelCallback, export_cancel, NULL);
+  XtAddCallback(file, XmNokCallback, export_it, NULL);
+  XtManageChild(file);
+
+}    
 /* this function manages the popup at the default button */
 void popup_callback(Widget button, XtPointer client_data, XtPointer
 		     call_data)
 {
     int selected = (int) client_data;
-    static Widget file;
     Widget default_button, message_box;
     XmString message;
 
@@ -816,13 +943,19 @@ void popup_callback(Widget button, XtPointer client_data, XtPointer
 	  preferences_dialog();
 	  break;
 	case 5:
+	  export(default_button);
+	  break;
+	case 6:
 	  /* open the information dialog */
 	  message_box = XmCreateInformationDialog(default_button,
 						  "Message_Box",
 						  NULL,
 						  0);
-	  message = XmStringLtoRCreate("Toolbar 1.1 by Thomas Zwettler", XmSTRING_DEFAULT_CHARSET);
-	  XtVaSetValues(message_box, 
+
+	  message = XmStringLtoRCreate("Toolbar 1.2 by Thomas Zwettler\nSend bugreports or comments to\nspiff@calvin.priv.at", XmSTRING_DEFAULT_CHARSET);
+	  XtVaSetValues(message_box,  
+			XmNdialogStyle,
+			XmDIALOG_PRIMARY_APPLICATION_MODAL,
 			XmNmessageString,
 			message,
 			XmNdialogTitle,
@@ -831,6 +964,38 @@ void popup_callback(Widget button, XtPointer client_data, XtPointer
 			screen_width/2,
 			XmNy,
 			screen_height/2,
+			XmNmessageAlignment,
+			XmALIGNMENT_CENTER,
+			NULL);
+
+	  XtUnmanageChild(XmMessageBoxGetChild(message_box,
+					       XmDIALOG_CANCEL_BUTTON));
+	  XtUnmanageChild(XmMessageBoxGetChild(message_box,
+					       XmDIALOG_HELP_BUTTON));
+	  XmStringFree(message);
+	  XtManageChild(message_box);
+	  break;
+	case 7:
+	  /* no help at the moment */
+	  message_box = XmCreateInformationDialog(default_button,
+						  "Message_Box",
+						  NULL,
+						  0);
+
+	  message = XmStringLtoRCreate("Sorry, no help available at the moment", 
+				       XmSTRING_DEFAULT_CHARSET);
+
+	  XtVaSetValues(message_box, 
+			XmNdialogStyle,
+			XmDIALOG_PRIMARY_APPLICATION_MODAL,
+			XmNmessageString,
+			message,
+			XmNdialogTitle,
+			XmStringCreateLocalized("Help"),
+			XmNx,
+			screen_width/2,
+			XmNy,
+			screen_height/2,
 			NULL);
 
 	  XtUnmanageChild(XmMessageBoxGetChild(message_box,
@@ -840,9 +1005,7 @@ void popup_callback(Widget button, XtPointer client_data, XtPointer
 	  XmStringFree(message);
 	  XtManageChild(message_box);
 	  break;
-	case 6:
-	  break;
-	case 7:
+	case 8:
 	  /* exit the program */
 	  
 	  if(save_on_exit)
