@@ -8,6 +8,7 @@
 #include <string.h>
 #include <signal.h>
 #include <assert.h>   
+#include <getopt.h>
 
 extern void group_popup_callback();
 extern void group_callback();
@@ -19,6 +20,8 @@ void get_def_pixmap(Widget parent);
 void button_event(Widget, XtPointer, XEvent);
 void popup_callback(Widget, XtPointer, XtPointer);
 void new_popup_callback(Widget, XtPointer, XtPointer);
+void show_bar(Widget, XtPointer, XEvent *);
+void hide_bar(Widget, XtPointer, XEvent *);
 
 Screen *screen;
 Display *display;
@@ -37,6 +40,12 @@ int save_on_exit = False;
 int stay_on_top = True;
 int group = NORMAL;
 int quick_info = True;
+int auto_hide = True;
+int childs_open = 0;
+
+Position old_y;
+
+int not_again = True;
 
 extern char *optarg;
 extern int optind, opterr, optopt;
@@ -65,6 +74,7 @@ void button_pressed(Widget button, XtPointer client_data, XtPointer
 		     call_data)
 {
     Position x_pos, y_pos;
+    int pos_index;
 
     if( !expand )
 	{
@@ -75,6 +85,7 @@ void button_pressed(Widget button, XtPointer client_data, XtPointer
 			XmNwidth,
 			&window_width,
 			NULL);
+
 	  
 	  if(position == TOPRIGHT)
 	    XtVaSetValues(toplevel,
@@ -83,6 +94,7 @@ void button_pressed(Widget button, XtPointer client_data, XtPointer
 			  XmNy,
 			  0,
 			  NULL);
+
 	  if(position == BOTTOMRIGHT)
 	    XtVaSetValues(toplevel,
 			  XmNx,
@@ -90,6 +102,7 @@ void button_pressed(Widget button, XtPointer client_data, XtPointer
 			  XmNy,
 			  screen_height - button_y,
 			  NULL);	
+
 	  if(position == BOTTOMLEFT)
 	    XtVaSetValues(toplevel,
 			  XmNx,
@@ -97,6 +110,7 @@ void button_pressed(Widget button, XtPointer client_data, XtPointer
 			  XmNy,
 			  screen_height - button_y,
 			  NULL);	
+
  	  if(position == TOPLEFT)
 	    XtVaSetValues(toplevel,
 			  XmNx,
@@ -104,26 +118,43 @@ void button_pressed(Widget button, XtPointer client_data, XtPointer
 			  XmNy,
 			  0,
 			  NULL);
+
+	  XtVaGetValues(button,
+			XmNpositionIndex,
+			&pos_index,
+			NULL);
+
  	  if(position == CUSTOM)
-	    XtVaSetValues(toplevel,
-			  XmNx,
-			  START_X,
-			  XmNy,
-			  START_Y,
-			  NULL);
-			   
+	    {
+	      /*if((pos_index == XmLAST_POSITION) && (orientation ==
+						    XmHORIZONTAL))*/
+		XtVaSetValues(toplevel,
+			      XmNx,
+			      START_X,
+			      XmNy,
+			      START_Y,
+			      NULL);
+		/*else
+		XtVaSetValues(toplevel,
+			      XmNx,
+			      START_X + window_width - button_x,
+			      XmNy,
+			      START_Y,
+			      NULL);*/
+	    }
+
 	  XtVaSetValues(button,
 			XmNpositionIndex,
 			0,
 			NULL);
 	  
-	  
 	  XtVaGetValues(start->button,
 			XmNwidth,
 			&button_x,
 			XmNheight,
 			&button_y,
 			NULL);
+
 	  XtVaSetValues(toplevel,  
 			XmNheight, 
 			button_y,
@@ -139,7 +170,7 @@ void button_pressed(Widget button, XtPointer client_data, XtPointer
 		XtUnmanageChild(XtParent(toolbar->rowcol));
 	      toolbar = toolbar->next;
 	    }
-		
+	  childs_open = 0;
 	  expand = 1;
 	}
     else
@@ -153,6 +184,7 @@ void button_pressed(Widget button, XtPointer client_data, XtPointer
 			      XmLAST_POSITION,
 			      NULL);
 
+
 	    XtVaGetValues(toplevel,
 			  XmNx,
 			  &x_pos,
@@ -160,10 +192,22 @@ void button_pressed(Widget button, XtPointer client_data, XtPointer
 			  &y_pos,
 			  NULL);
 
+	    if((x_pos > (screen_width/2)) &&
+	       (orientation==XmHORIZONTAL))
+	      {
+		XtVaSetValues(button,
+			      XmNpositionIndex,
+			      XmLAST_POSITION,
+			      NULL);
+		x_pos = START_X - window_width + button_x;
+	      }
+
+
 	    if(((y_pos+window_height) > screen_height)&&(orientation==XmVERTICAL)) 
  		y_pos = screen_height-window_height; 
  	    if(((x_pos+window_width) > screen_width)&&(orientation==XmHORIZONTAL)) 
  		x_pos = screen_width-window_width; 
+		
 
 	    XtVaSetValues(toplevel,  
 			  XmNheight, 
@@ -175,6 +219,7 @@ void button_pressed(Widget button, XtPointer client_data, XtPointer
  			  XmNy, 
  			  y_pos, 
 			  NULL ); 
+
 	    expand = 0;
 	}
 
@@ -261,7 +306,6 @@ Widget add_button(Widget row_col, tool_button *button)
 {
     Widget new_button, popup;
     Pixmap pixmap;
-    int width, height;
     int pos_index;
     Position x_pos, y_pos;
 
@@ -283,10 +327,12 @@ Widget add_button(Widget row_col, tool_button *button)
 
     button->rowcol = row_col;
     
-    if(((orientation == XmHORIZONTAL) && (position == BOTTOMRIGHT)) || 
-       ((orientation == XmVERTICAL) && (position == BOTTOMRIGHT)) ||
-       ((orientation == XmHORIZONTAL) && (position == TOPRIGHT)) ||
-       ((orientation == XmVERTICAL) && (position == BOTTOMLEFT)))
+    if((((orientation == XmHORIZONTAL) && (position == BOTTOMRIGHT)) || 
+	((orientation == XmVERTICAL) && (position == BOTTOMRIGHT)) ||
+	((orientation == XmHORIZONTAL) && (position == TOPRIGHT)) ||
+	((orientation == XmVERTICAL) && (position == BOTTOMLEFT)) ||
+	((orientation == XmHORIZONTAL) && (position == CUSTOM))) &&
+       (START_X > (screen_width/2)))
       pos_index = 0;
     else
       pos_index = XmLAST_POSITION;
@@ -384,6 +430,10 @@ Widget add_button(Widget row_col, tool_button *button)
 					  XmVaPUSHBUTTON,
 					  XmStringCreateLocalized("Preferences"), 
 					  'r',
+					  NULL, NULL,
+					  XmVaPUSHBUTTON,
+					  XmStringCreateLocalized("Export config"), 
+					  'E',
 					  NULL, NULL,     	      
 					  XmVaSEPARATOR,
 					  XmVaPUSHBUTTON,
@@ -465,10 +515,10 @@ Widget add_button(Widget row_col, tool_button *button)
 int read_config(void)
 {
     struct stat file;
-    tool_button *button, *help;
+    tool_button *button, *help = NULL;
     int config;
     char filename[MAXPATH];
-    int ok = 0, counter = 0, pos;
+    int counter = 0, pos;
 
     strcpy(filename, getenv("HOME"));
     strcat(filename, "/");
@@ -541,6 +591,9 @@ int read_config(void)
 	fprintf(stderr, "read START_Y failed\n");  
     }
 
+    if(read(config, (char *)&auto_hide, sizeof(int)) != sizeof(int))
+      fprintf(stderr, "read auto hide failed\n");  
+
     strcpy(def_pix, start->pixmap_path);
     
     toolbar = help;
@@ -569,18 +622,12 @@ void handler( String msg )
 int main(int argc, char *argv[])
 {
 
-    Widget popup, button, group, row_col;
+    Widget group, row_col;
     XtAppContext app;
-    int height, width;
-    int depth, config, counter, layout;
-    Pixmap button_pixmap;
+    int config, counter, layout;
     Position x, y;
     int c, flag=0;
 
-    int i, save_xypos = 1;
-    int startx_hold, starty_hold;
-
-
     button_x = 0;
     button_y = 0;
     
@@ -769,6 +816,7 @@ int main(int argc, char *argv[])
 		  &button_y,
 		  NULL);
 
+    
     XtRealizeWidget(toplevel);
 
 
@@ -793,6 +841,7 @@ int main(int argc, char *argv[])
 		    XmNy,
 		    0,
 		    NULL);
+
     if(position == BOTTOMLEFT) 
       XtVaSetValues(toplevel,
 		    XmNx,
@@ -800,6 +849,7 @@ int main(int argc, char *argv[])
 		    XmNy,
 		    screen_height - window_height,
 		    NULL);
+
     if(position == BOTTOMRIGHT) 
       XtVaSetValues(toplevel,
 		    XmNx,
@@ -807,20 +857,140 @@ int main(int argc, char *argv[])
 		    XmNy,
 		    screen_height - window_height,
 		    NULL);
-    if(position == CUSTOM) 
-      XtVaSetValues(toplevel,
-		    XmNx,
-		    START_X,
-		    XmNy,
-		    START_Y,
-		    NULL);
 
+    if(position == CUSTOM) 
+      {
+	if((orientation == XmHORIZONTAL) && (START_X >
+					     (screen_width/2)))
+	        XtVaSetValues(toplevel,
+			      XmNx,
+			      START_X - window_width + button_x,
+			      XmNy,
+			      START_Y,
+			      NULL);
+	else
+	  XtVaSetValues(toplevel,
+			XmNx,
+			START_X,
+			XmNy,
+			START_Y,
+			NULL);
+      }
+
+    XtAddEventHandler(toplevel, EnterWindowMask, False,
+		      (XtEventHandler) show_bar, NULL);
+    XtAddEventHandler(toplevel, LeaveWindowMask, False,
+		      (XtEventHandler) hide_bar, NULL);
+    
     if(stay_on_top)
       XRaiseWindow(XtDisplay(toplevel), XtWindow(toplevel));
     else
       XLowerWindow(XtDisplay(toplevel), XtWindow(toplevel));
 
     XtAppMainLoop(app);
+    return 0;
+}
+
+/* this function moves the toolbar out of the screen except of 3
+   pixels so it can be activated again */
+void hide_bar(Widget button, XtPointer client_data, XEvent *event)
+{
+  Position y_pos;
+
+  XtVaGetValues(toplevel,
+		XmNy,
+		&old_y,
+		NULL);
+
+  if(!not_again)
+    return;
+
+  if(auto_hide == False)
+    return;
+
+  /* auto hide only when displayed at the screen borders */
+  if(((position == CUSTOM) && (old_y != 0)) &&
+     ((position == CUSTOM) && (old_y != screen_height)))
+    return;
+
+  /* if any group button displays other buttons then don't hide
+     until they are all closed */
+  if(childs_open > 0)
+    return;
+
+  not_again = False;
+  y_pos = old_y;
+  
+  if(y_pos > (screen_height/2))
+    while(y_pos != (screen_height - 3))
+      {
+	y_pos++;
+	XtVaSetValues(toplevel,
+		      XmNy,
+		      y_pos,
+		      NULL);
+	XmUpdateDisplay(toplevel);
+      }
+  else
+    {
+      if(orientation == XmHORIZONTAL)
+	y_pos += button_y;
+      else
+	y_pos += window_height;
+      while(y_pos > 3)
+	{
+	  y_pos--;
+	  if(orientation == XmHORIZONTAL)
+	    XtVaSetValues(toplevel,
+			  XmNy,
+			  y_pos - button_y,
+			  NULL);
+	  else
+	    XtVaSetValues(toplevel,
+			  XmNy,
+			  y_pos - window_height,
+			  NULL);	    
+	  XmUpdateDisplay(toplevel);
+	}
+    }
+}
+
+void show_bar(Widget button, XtPointer client_data, XEvent *event)
+{
+  Position y_pos;
+
+  
+  if(not_again)
+    return;
+
+  not_again = True;
+
+  XtVaGetValues(toplevel,
+		XmNy,
+		&y_pos,
+		NULL);
+
+  if(old_y > (screen_height/2))
+    while(y_pos != old_y)
+      {
+	y_pos--;
+	XtVaSetValues(toplevel,
+		      XmNy,
+		      y_pos,
+		      NULL);
+	XmUpdateDisplay(toplevel);
+      }
+  else
+    while(y_pos < old_y)
+      {
+	y_pos++;
+	XtVaSetValues(toplevel,
+		      XmNy,
+		      y_pos,
+		      NULL);
+	XmUpdateDisplay(toplevel);
+      }
+
 }
 
 Pixmap xmtGetPixmapByDepth( Screen *screen,                          /* andre */
@@ -927,8 +1097,6 @@ Pixmap xmtGetPixmapByDepth( Screen *screen,                          /* andre */
                                   substitutions, XtNumber(substitutions),
                                   NULL );
 
-  /*printf( ">>%s<<\n", image_path ); /**/
-
   if( image_path )
     {
 
